package ACL2::Lisp::List;
use base 'ACL2::Lisp::BaseNode';

#
#  AST node for lists
#  Copyright (C) 2009 Antonio García Domínguez
#  $Id$
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

sub new {
    my ($pkg, $value, $source) = @_;

    if (!defined($source)) {
        my (@elems) = map { $_->source } @$value;
        $source = '(' . (join '', @elems) . ')';
    }

    return ACL2::Lisp::BaseNode::new($pkg, $value, $source);
}

sub clean_tree {
    my ($self) = @_;
    my @elems  = map {$_->clean_tree} @{$self->value};
    return ACL2::Lisp::List->new(\@elems, $self->source);
}

=head2 length($self)

Returns the number of elements of the list.

=cut

sub length {
    return scalar @{$_[0]->value};
}

=head2 max_depth($self)

Returns the maximum depth of the list. For instance:

=over 2

=item *

C<()> returns 0 (no elements).

=item *

C<(x)> returns 1.

=item *

C<(x (y z))> returns 2.

=back

=cut

sub max_depth
{
    my ($this) = @_;
    my $depth  = 0;

    for (@{$this->value}) {
        $_->type eq "list" && do {
            my $depth_with_sl = $_->max_depth + 1;
            $depth = $depth_with_sl
                if $depth_with_sl > $depth;
            next;
        };

        $_->type ne "whitespace" && $_->type ne "comment" && do {
            $depth = 1 unless $depth > 1;
            next;
        };
    }

    return $depth;
}


1;
