package ACL2::Proceso::AugmentWithHints;
use base ACL2::Proceso;

=head1 NAME

ACL2::Proceso::AugmentWithHints - augment the goals with hypothesis from a hint

=head1 FUNCTIONS

=cut

#
#    AugmentWithHints: augment the goals with hypothesis from a hint
#    Copyright (C) 2009 Antonio García Domínguez
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use utf8;

use ACL2::Orden;
use ACL2::Proceso::Fabrica;

BEGIN {
    ACL2::Proceso::Fabrica->registrar(
        __PACKAGE__,
        sub { return ACL2::Proceso::AugmentWithHints->_identificar(@_); });
}

sub escribir_xml_cabecera {
    my ($this, $writer) = @_;

    $writer->startTag(ACL2::Tags::AUGMENT_TAG);

    $writer->startTag(ACL2::Tags::AUGMENT_SOURCES_TAG);
    while (my ($source, $method) = each(%{$this->{sources}})) {
        $writer->emptyTag(ACL2::Tags::AUGMENT_SOURCE_TAG,
                          name   => $source,
                          method => $method);
    }
    $writer->endTag(ACL2::Tags::AUGMENT_SOURCES_TAG);

    ACL2::Orden->salida_parrafos($writer, $this->{salida});
}

sub escribir_xml_final {
    my ($this, $writer) = @_;
    $writer->endTag(ACL2::Tags::AUGMENT_TAG);
}

sub _identificar
{
    my ($paquete, $salida) = @_;
    my %res      = ();

    $salida =~ /^
        # This part uniquely identifies this method
        (?-x:We augment the goal with the hypothes[ie]s provided)
        (?-x: by the :USE hint\.)
        # Obtain the sources from which we can derive these hypotheses
        (?-x: (?:These hypotheses|The hypothesis) can be derived from (.*)\.\s*We are left)/ox
        or return undef;

    my $sources_text = substr $salida, $-[1], $+[1] - $-[1];
    $res{sources}    = { map { split /\s*via\s*/, $_ } (
                             split /\s*(?:,|and)\s*/, $sources_text
                         ) };

    return \%res;
}

1;
